#!/usr/bin/perl -w
#
# $Id: dns_restart.pl,v 1.4.6.1 2001/11/30 02:07:14 ssmith Exp $
#
# starts, stops, and restarts a service on demand, with some extra
# safety checks.  jm.

# configure here: (mostly)
my $SERVICE = "named";	# name of initd script for this daemon
my $CMDLINE = "named";  # contents of /proc/nnn/cmdline for this daemon
my $RESTART = "reload"; # restart action

use lib qw( /usr/sausalito/perl );
use FileHandle;
use Sauce::Util;
use Getopt::Long;
use CCE;
$cce = new CCE;

my $DEBUG = 0;
GetOptions("debug" => \$DEBUG);

if ($DEBUG) {
	$cce->connectuds()
} else {
	$cce->connectfd()
}

my ($sysoid) = $cce->find("System");
my ($ok, $obj) = $cce->get($sysoid, "DNS");

# fix chkconfig information:
if ($obj->{enabled}) {
  system ("/sbin/chkconfig", "--add", $SERVICE);
} else {
  system ("/sbin/chkconfig", "--del", $SERVICE);
}

# check to see if the service is presently running;
my $running = 0;
{
  my $fh = new FileHandle("</var/run/${SERVICE}.pid");
  if ($fh) {
    my $pid = <$fh>; chomp($pid);
    $DEBUG && print STDERR "old $SERVICE pid = $pid\n";
    $fh->close();
    
    $fh = new FileHandle("</proc/${pid}/cmdline");
    if ($fh) {
      my $cmdline = <$fh>; chomp($cmdline);
      $DEBUG && print STDERR "old $SERVICE cmdline = $cmdline\n";
      $fh->close();
      
      if ($cmdline =~ m/${CMDLINE}$/) { $running = 1; }
    }
  }
}

# do the right thing
if (!$running && $obj->{enabled}) {
  system("/etc/rc.d/init.d/${SERVICE}", "start");
  sleep(1); # wait for named to really start
}
elsif ($running && !$obj->{enabled}) {
  system("/etc/rc.d/init.d/${SERVICE}", "stop");
}
elsif ($running && $obj->{enabled}) {
  system("/etc/rc.d/init.d/${SERVICE}", $RESTART);
  sleep(1); # wait for named to really start
}

# is it running now?
$running = 0;
{
  my $fh = new FileHandle("</var/run/${SERVICE}.pid");
  if ($fh) {
    my $pid = <$fh>; chomp($pid);
    $DEBUG && print STDERR "new $SERVICE pid = $pid\n";
    $fh->close();
    
    $fh = new FileHandle("</proc/${pid}/cmdline");
    if ($fh) {
      my $cmdline = <$fh>; chomp($cmdline);
      $DEBUG && print STDERR "new $SERVICE name = $cmdline\n";
      $fh->close();
      
      if ($cmdline =~ m/${CMDLINE}$/) { $running = 1; }
    }
  }
}

# report the did-not-start error, if necessary:
if ($obj->{enabled} && !$running) {
  $cce->warn("[[base-dns.${SERVICE}-did-not-start]]");
  $cce->bye("FAIL");
  $DEBUG && print "ERROR: dns failed to start!\n";
  exit 1;
} else {
  $cce->bye("SUCCESS");
  exit 0;
}
