#!/usr/bin/perl -w

use lib qw(/usr/sausalito/perl);
use CCE;
my $cce = new CCE;
$cce->connectfd();

my $oid = $cce->event_oid();
my $obj = $cce->event_object();

if (defined($obj->{ipaddr}) && defined($obj->{netmask})) {
  my $ipbin = pack("C4", split(/\./, $obj->{ipaddr}));
  my $ipnm = pack("C4", parse_netmask($obj->{netmask}));
  my $nbits = netmask_to_netbits($obj->{netmask});
  my $network = join(".",unpack("C4", $ipbin & $ipnm)) . "/" . $nbits;
  $cce->set($oid, "", { "network" => $network } );
}

$cce->bye("SUCCESS");
exit(0);

sub parse_netmask
{
  my $nbits = shift;
  if ($nbits =~ m/^\s*\d+\s*$/) {
    return unpack("C4",pack("B32", '1' x $nbits . '0' x (32 - $nbits) ));
  };
  if ($nbits =~ m/^\s*(\d+)\.(\d+)\.(\d+)\.(\d+)\s*$/) {
    return ($1,$2,$3,$4);
  }
  warn ("Invalid netmask: $nbits\n");
  return (255,255,255,255);
}

# netmask_to_netbits
#
# convert generalized netmask format to just a simple bit-count.
########################################
sub netmask_to_netbits
{
  my $nbits = shift;
  if ($nbits =~ m/^\s*(\d+)\s*$/) {
    return $1;
  };
  if ($nbits =~ m/^\s*(\d+)\.(\d+)\.(\d+)\.(\d+)\s*$/) {
    my @bits = split(//, unpack("B32", pack("C4",$1,$2,$3,$4)));
    $nbits = 0;
    while (@bits) { $nbits += shift(@bits); }
    return $nbits;
  }
  warn ("Invalid netmask: $nbits\n");
  return (32);
}
